from .chunk import Chunk
from .converter import GenericConverter
from .knowledge import (
    EmbeddingModelEnum,
    GithubFileSourceConfig,
    GithubRepoSourceConfig,
    Knowledge,
    KnowledgeCreate,
    KnowledgeSourceEnum,
    KnowledgeSplitConfig,
    KnowledgeTypeEnum,
    S3SourceConfig,
    TextSourceConfig,
)
from .knowledge_create import (
    GithubRepoCreate,
    ImageCreate,
    JSONCreate,
    MarkdownCreate,
    PDFCreate,
    QACreate,
    TextCreate,
)
from .page import PageParams, PageResponse, StatusStatisticsPageResponse
from .retrieval import (
    RetrievalByKnowledgeRequest,
    RetrievalBySpaceRequest,
    RetrievalChunk,
    RetrievalRequest,
)
from .space import Space, SpaceCreate, SpaceResponse
from .splitter import (
    BaseCharSplitConfig,
    GeaGraphSplitConfig,
    JSONSplitConfig,
    MarkdownSplitConfig,
    PDFSplitConfig,
    TextSplitConfig,
    YuqueSplitConfig,
)
from .task import Task, TaskRestartRequest, TaskStatus
from .tenant import Tenant

__all__ = [
    "Chunk",
    "KnowledgeSourceEnum",
    "KnowledgeTypeEnum",
    "EmbeddingModelEnum",
    "KnowledgeSplitConfig",
    "KnowledgeCreate",
    "TextCreate",
    "ImageCreate",
    "JSONCreate",
    "MarkdownCreate",
    "PDFCreate",
    "GithubRepoCreate",
    "QACreate",
    "GithubRepoSourceConfig",
    "GithubFileSourceConfig",
    "S3SourceConfig",
    "TextSourceConfig",
    "Knowledge",
    "Space",
    "SpaceCreate",
    "SpaceResponse",
    "PageParams",
    "PageResponse",
    "StatusStatisticsPageResponse",
    "RetrievalBySpaceRequest",
    "RetrievalByKnowledgeRequest",
    "RetrievalChunk",
    "RetrievalRequest",
    "Task",
    "TaskStatus",
    "TaskRestartRequest",
    "Tenant",
    "GenericConverter",
    "BaseCharSplitConfig",
    "JSONSplitConfig",
    "MarkdownSplitConfig",
    "PDFSplitConfig",
    "TextSplitConfig",
    "GeaGraphSplitConfig",
    "YuqueSplitConfig",
]



import json
from typing import List

from langchain_text_splitters import RecursiveJsonSplitter

from whiskerrag_types.interface.splitter_interface import BaseSplitter
from whiskerrag_types.model.knowledge import JSONSplitConfig
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.SPLITTER, "json")
class JSONSplitter(BaseSplitter[JSONSplitConfig, Text]):

    def split(self, content: Text, split_config: JSONSplitConfig) -> List[Text]:
        """Splits JSON content into smaller chunks based on the provided configuration."""
        json_content = {}
        try:
            json_content = json.loads(content.content)
            if not isinstance(json_content, dict):
                raise ValueError("JSON content must be a dictionary.")
        except json.JSONDecodeError:
            raise ValueError("Invalid JSON content provided for splitting.")
        except ValueError as e:
            raise ValueError(f"Error processing JSON content: {str(e)}")
        splitter = RecursiveJsonSplitter(
            max_chunk_size=split_config.max_chunk_size,
            min_chunk_size=split_config.min_chunk_size,
        )
        split_texts = splitter.split_json(json_content)
        return [
            Text(content=str(text), metadata=content.metadata) for text in split_texts
        ]

    def batch_split(
        self, content: List[Text], split_config: JSONSplitConfig
    ) -> List[List[Text]]:
        return [self.split(text, split_config) for text in content]



from typing import List

from langchain_text_splitters import RecursiveCharacterTextSplitter

from whiskerrag_types.interface.splitter_interface import BaseSplitter
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_types.model.splitter import YuqueSplitConfig
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.SPLITTER, "yuque")
class BaseSplitterRegistry(BaseSplitter[YuqueSplitConfig, Text]):

    def split(self, content: Text, split_config: YuqueSplitConfig) -> List[Text]:
        separators = split_config.separators or [
            # First, try to split along Markdown headings (starting with level 2)
            "\n#{1,6} ",
            # Note the alternative syntax for headings (below) is not handled here
            # Heading level 2
            # ---------------
            # End of code block
            "```\n",
            # Horizontal lines
            "\n\\*\\*\\*+\n",
            "\n---+\n",
            "\n___+\n",
            # Note that this splitter doesn't handle horizontal lines defined
            # by *three or more* of ***, ---, or ___, but this is not handled
            "\n\n",
            "\n",
            " ",
        ]
        splitter = RecursiveCharacterTextSplitter(
            chunk_size=split_config.chunk_size,
            chunk_overlap=split_config.chunk_overlap,
            separators=separators,
            keep_separator=False,
        )
        split_texts = splitter.split_text(content.content)
        return [Text(content=text, metadata=content.metadata) for text in split_texts]

    def batch_split(
        self, content_list: List[Text], split_config: YuqueSplitConfig
    ) -> List[List[Text]]:
        return [self.split(content, split_config) for content in content_list]



from typing import List

from langchain_community.document_loaders.yuque import YuqueLoader

from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.model.knowledge import KnowledgeSourceEnum, YuqueSourceConfig
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.KNOWLEDGE_LOADER, KnowledgeSourceEnum.YUQUE)
class WhiskerYuqueLoader(BaseLoader[Text]):

    async def load(self) -> List[Text]:
        if not isinstance(self.knowledge.source_config, YuqueSourceConfig):
            raise AttributeError("Invalid source config type for YuqueLoader")
        text_list: List[Text] = []
        try:
            loader = YuqueLoader(
                access_token=self.knowledge.source_config.auth_info,
                api_url=self.knowledge.source_config.api_url,
            )
            content = None
            # Extract book_id and document_id from source_config
            book_id = self.knowledge.source_config.book_id
            document_id = self.knowledge.source_config.document_id
            # Check if book_id and document_id are provided
            if book_id is not None and document_id is not None:
                try:
                    document = loader.get_document(int(book_id), int(document_id))
                    parsed_document = loader.parse_document(document)
                    content = loader.parse_document(document)
                    text_list.append(
                        Text(
                            content=parsed_document.page_content,
                            metadata=parsed_document.metadata,
                        )
                    )
                except Exception as e:
                    raise ValueError(f"Failed to get document: {e}")
            # Check if only book_id is provided
            elif book_id is not None:
                try:
                    documents = loader.get_books(user_id=loader.get_user_id())
                    for document in documents:
                        document_id = document.get("id")
                        if document_id is None:
                            print("Document ID is None, skipping...")
                            continue
                        document = loader.get_document(int(book_id), int(document_id))
                        parsed_document = loader.parse_document(document)
                        content = loader.parse_document(document)
                        text_list.append(
                            Text(
                                content=parsed_document.page_content,
                                metadata=parsed_document.metadata,
                            )
                        )
                except Exception as e:
                    raise ValueError(f"Failed to get book: {e}")

            if content is None:
                raise ValueError("No content found for the specified parameters")

            return text_list

        except Exception as e:
            raise Exception(f"Failed to load content from Yuque: {e}")



from abc import ABC, abstractmethod
from typing import List, Optional

from ..interface import DBPluginInterface, LoggerManagerInterface, SettingsInterface
from ..model import Knowledge, Task, Tenant


class TaskEnginPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface
    db_plugin: Optional[DBPluginInterface] = None

    def __init__(
        self,
        logger: LoggerManagerInterface,
        settings: SettingsInterface,
    ):
        self.settings = settings
        self.logger = logger
        self._initialized: bool = False

    async def ensure_initialized(self, db_plugin: Optional[DBPluginInterface]) -> None:
        if not self._initialized:
            try:
                self.logger.info("TaskEngine plugin is initializing...")
                await self.init()
                if db_plugin:
                    if not db_plugin.is_initialized:
                        self.logger.info("Initializing DB plugin...")
                        await db_plugin.ensure_initialized()
                    self.db_plugin = db_plugin
                self._initialized = True
                self.logger.info("TaskEngine plugin is initialized")
            except Exception as e:
                self.logger.error(f"TaskEngine plugin init error: {e}")
                raise

    @property
    def is_initialized(self) -> bool:
        return self._initialized

    @abstractmethod
    async def init(self) -> None:
        """
        Initialize the task engine plugin, such as loading middleware, establishing contact with the task execution engine, etc.
        """
        pass

    @abstractmethod
    async def init_task_from_knowledge(
        self, knowledge_list: List[Knowledge], tenant: Tenant
    ) -> List[Task]:
        """
        Initialize a list of tasks from the knowledge list.
        """
        pass

    @abstractmethod
    async def batch_execute_task(
        self, task_list: List[Task], knowledge_list: List[Knowledge]
    ) -> List[Task]:
        """
        Execute a list of tasks.
        """
        pass



from typing import List, Optional, Union

from deprecated import deprecated
from pydantic import BaseModel, Field, field_serializer

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import EmbeddingModelEnum


class RetrievalBaseConfig(BaseModel):
    embedding_model_name: Union[EmbeddingModelEnum, str] = Field(
        ..., description="The name of the embedding model"
    )
    similarity_threshold: float = Field(
        0.5,
        ge=0.0,
        le=1.0,
        description="The similarity threshold, ranging from 0.0 to 1.0.",
    )
    top: int = Field(1024, ge=1, description="The maximum number of results to return.")
    metadata_filter: dict = Field({}, description="metadata filter")

    @field_serializer("embedding_model_name")
    def serialize_embedding_model_name(
        self, embedding_model_name: Optional[Union[EmbeddingModelEnum, str]]
    ) -> Optional[str]:
        if embedding_model_name:
            if isinstance(embedding_model_name, EmbeddingModelEnum):
                return embedding_model_name.value
            else:
                return embedding_model_name
        else:
            return None


class QueryBySpaceConfig(RetrievalBaseConfig):
    type: str = Field(
        "query_in_space_list",
        description="The type of the request, should be 'query_in_space_list'.",
    )
    space_id_list: List[str] = Field(..., description="space id list")


class QueryByKnowledgeConfig(RetrievalBaseConfig):
    type: str = Field(
        "query_in_knowledge_list",
        description="The type of the request, should be 'query_in_knowledge_list'.",
    )
    embedding_model_name: str = Field(
        ..., description="The name of the embedding model"
    )
    space_id_list: List[str] = Field(..., description="knowledge id list")


class QueryByDeepRetrieval(RetrievalBaseConfig):
    type: str = Field(
        "deep_retrieval",
        description="The type of the request, should be 'deep_retrieval'.",
    )
    space_name_list: List[str] = Field(..., description="space name list")


RetrievalConfig = Union[
    QueryBySpaceConfig, QueryByKnowledgeConfig, QueryByDeepRetrieval, dict
]


@deprecated(
    "RetrievalBySpaceRequest is deprecated, please use RetrievalRequest instead."
)
class RetrievalBySpaceRequest(RetrievalBaseConfig):
    question: str = Field(..., description="The query question")
    space_id_list: List[str] = Field(..., description="space id list")


@deprecated(
    "RetrievalBySpaceRequest is deprecated, please use RetrievalRequest instead."
)
class RetrievalByKnowledgeRequest(RetrievalBaseConfig):
    question: str = Field(..., description="The query question")
    knowledge_id_list: List[str] = Field(..., description="knowledge id list")


class RetrievalRequest(BaseModel):
    question: str = Field(..., description="The query question")
    config: RetrievalConfig = Field(
        ..., description="The configuration for the retrieval request"
    )


class RetrievalChunk(Chunk):
    similarity: float = Field(
        ..., description="The similarity of the chunk, ranging from 0.0 to 1.0."
    )
